from typing import TYPE_CHECKING
import uuid
from sqlmodel import SQLModel, Field, Relationship, Session
import pandas as pd
import os as os

if TYPE_CHECKING:
    from ._team import Team
    from ._trainer import Trainer


class Player(SQLModel, table=True):
    player_id: uuid.UUID = Field(
        default_factory=uuid.uuid4,
        primary_key=True,
        unique=True,
        index=True,
        sa_column_kwargs={"comment": "Unique identifier for the player"},
    )
    name: str
    age: int
    weight: int
    height: float
    salary: float
    position: str
    pac: float
    sho: float
    pas: float
    dri: float
    defe: float
    phy: float
    goalkeeping: float

    team_id: uuid.UUID | None = Field(default=None, foreign_key="team.team_id")
    team: "Team" = Relationship(back_populates="players")

    trainer_id: uuid.UUID | None = Field(default=None, foreign_key="trainer.trainer_id")
    trainer: "Trainer" = Relationship(back_populates="players")

    @staticmethod
    def create_player(player: "Player", csv_name: str, headers: list, engine) -> None:
        """
        Create a .csv with the player's categories if it doesn't exist. Add the player to the csv and upload to database

        Args:
            csv_name: str: csv's name
            headers: list: header's columns
            Player
            engine
        """
        session = Session(engine)

        if not os.path.exists(csv_name):
            df = pd.DataFrame(columns=headers)

            df.to_csv(csv_name, index=False)

        df = pd.read_csv(csv_name)

        new_player = pd.DataFrame(
            [[getattr(player, attr) for attr in Player.__fields__.keys()]],
            columns=df.columns,
        )

        df = pd.concat([df, new_player], ignore_index=True)

        df.to_csv(csv_name, index=False)

        player_instance = Player(**new_player.iloc[0].to_dict())

        session.add(player_instance)
        session.commit()



from typing import TYPE_CHECKING
import uuid
from sqlmodel import SQLModel, Field, Relationship, Session
import os as os
import pandas as pd

if TYPE_CHECKING:
    from ._player import Player
    from ._trainer import Trainer


class Team(SQLModel, table=True):
    team_id: uuid.UUID = Field(
        default_factory=uuid.uuid4,
        primary_key=True,
        unique=True,
        index=True,
        sa_column_kwargs={"comment": "Unique identifier for the team"},
    )
    name: str
    budget: float

    players: "Player" = Relationship(back_populates="team")
    trainer: "Trainer" = Relationship(back_populates="team")

    @staticmethod
    def create_team(team: "Team", csv_name: str, headers: list, engine) -> None:
        """
        Create a .csv with the team's categories if it doesn't exist. Add the team to the csv and upload to database

        Args:
            csv_name: str: csv's name
            headers: list: header's columns
            Team
            engine
        """
        session = Session(engine)

        if not os.path.exists(csv_name):
            df = pd.DataFrame(columns=headers)

            df.to_csv(csv_name, index=False)

        df = pd.read_csv(csv_name)

        new_team = pd.DataFrame(
            [[getattr(team, attr) for attr in Team.__fields__.keys()]],
            columns=df.columns,
        )

        df = pd.concat([df, new_team], ignore_index=True)

        df.to_csv(csv_name, index=False)

        team_instance = Team(**new_team.iloc[0].to_dict())

        session.add(team_instance)
        session.commit()



from ._player import Player
from ._team import Team
from ._trainer import Trainer


__all__ = [
    "Player",
    "Team",
    "Trainer",
    # "CenterBack",
    # "CenterRightBack",
    # "CenterLeftBack",
    # "SideFullBack",
    # "RightFullBack",
    # "LeftFullBack",
    # "DefensiveMidfielder",
    # "OfensiveMidfielder",
    # "SideMidfielder",
    # "RightSideMidfielder",
    # "LeftSideMidfielder",
    # "CentralStriker"
    # "SecondStriker",
    # "Winger",
    # "RightWinger",
    # "LeftWinger"
]



from decouple import config
from sqlmodel import create_engine, SQLModel, Session
from sqlmodel import Session

from schemas import *


def main():
    postgres_uri: str = config("POSTGRES_URI")
    engine = create_engine(postgres_uri)
    SQLModel.metadata.create_all(engine)

    with Session(engine) as session:
        """team_csv_name = "football_game/schemas/teams.csv"

        team_headers = [
            "team_id",
            "name",
            "budget",
        ]

        team_data = Team(
            name="Real Madrid",
            budget=99999
        )

        Team.create_team(
            team_data,
            team_csv_name,
            team_headers,
            engine
        )

        trainer_csv_name = "football_game/schemas/trainers.csv"

        trainer_headers = [
            "trainer_id",
            "name",
            "age",
            "salary",
            "team_id"
        ]

        trainer_data = Trainer(
            name="Ancelotti",
            age=59,
            salary=8491,
            budget=99999,
            #team=team
        )

        Trainer.create_trainer(
            trainer_data,
            trainer_csv_name,
            trainer_headers,
            engine
        )"""

        player_csv_name = "football_game/schemas/players.csv"

        player_headers = [
            "player_id",
            "name",
            "age",
            "weight",
            "height",
            "salary",
            "position",
            "pac",
            "sho",
            "pas",
            "dri",
            "defe",
            "phy",
            "goalkeeping",
            "team_id",
            "trainer_id",
        ]

        player_data = Player(
            name="Kepa Arrizabalaga",
            age=28,
            weight=88,
            height=1.88,
            salary=9000000,
            position="Goalkeeper",
            pac=33.5,
            sho=24.33,
            pas=41.67,
            dri=43.83,
            defe=18.4,
            phy=41,
            goalkeeping=85,
            # team=team,
            # trainer=trainer,
        )

        Player.create_player(player_data, player_csv_name, player_headers, engine)


if __name__ == "__main__":
    main()
